"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const base_1 = require("../base");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const teamUtils_1 = require("../utils/misc/teamUtils");
const initCalendar_1 = require("../utils/velocity/initCalendar");
const insertClosed_1 = require("../utils/velocity/insertClosed");
const insertControlData_1 = require("../utils/velocity/insertControlData");
class Control extends base_1.default {
    async run() {
        const userConfig = this.userConfig;
        const controlBuckets = [{
                key: '0_1',
                value: '1 day',
                color: '#f2929a',
                isInBucket: (value) => { return value <= 1; },
            }, {
                key: '2_5',
                value: '1 week',
                color: '#f2aea9',
                isInBucket: (value) => { return value > 1 && value <= 5; },
            }, {
                key: '6_10',
                value: '2 weeks',
                color: '#f1d0b7',
                isInBucket: (value) => { return value > 5 && value <= 10; },
            }, {
                key: '11_20',
                value: '1 month',
                color: '#d6e4c1',
                isInBucket: (value) => { return value > 10 && value <= 20; },
            }, {
                key: '21_40',
                value: '2 months',
                color: '#abdecc',
                isInBucket: (value) => { return value > 20 && value <= 40; },
            }, {
                key: '41+',
                value: 'more',
                color: '#bcc4de',
                isInBucket: (value) => { return value > 40; },
            }];
        for (const team of userConfig.teams) {
            const closedIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + '/cache/', team.name);
            console.log('Fetched ' + closedIssues.length + ' completed issues');
            const emptyCalendar = initCalendar_1.default(team.jqlHistory);
            const calendarWithClosed = await insertClosed_1.default(emptyCalendar, closedIssues);
            const calendarVelocity = {
                updatedAt: new Date().toJSON(),
                host: userConfig.jira.host,
                jqlCompletion: team.jqlCompletion,
                buckets: controlBuckets,
                weeks: insertControlData_1.default(calendarWithClosed, controlBuckets),
            };
            // Prune issues list from the object
            const trimmedPayload = Object.assign(Object.assign({}, calendarVelocity), { weeks: calendarVelocity.weeks.map((w) => {
                    if (w.completion.list !== undefined) {
                        w.completion.list = w.completion.list.map((i) => i.key);
                        // delete w.completion.list;
                    }
                    return w;
                }), open: (o) => {
                    if (o.list !== undefined) {
                        o.list = o.list.map((i) => i.key);
                        // delete o.list;
                    }
                    return o;
                } });
            const cacheDir = this.config.configDir + '/cache/';
            const controlFileStream = fs.createWriteStream(path.join(cacheDir, 'control-artifacts-' + teamUtils_1.getTeamId(team.name) + '.json'), { flags: 'w' });
            controlFileStream.write(JSON.stringify(trimmedPayload));
            controlFileStream.end();
        }
    }
}
exports.default = Control;
Control.description = 'Builds Control stats by week (how long issues were opened for)';
Control.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }) });
exports.formatDate = (dateString) => {
    const day = new Date(dateString);
    day.setUTCHours(12);
    day.setUTCMinutes(0);
    day.setUTCSeconds(0);
    return day;
};
